import logging
from copy import deepcopy

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Rados:
    """This module provides CLI interface to manage rados operations."""

    def __init__(self, base_cmd=""):
        self.base_cmd = f"{base_cmd}rados"

    def df(self, **kw):
        """
        The command stats will display the pools utilizations statistics.

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " df"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def lspools(self, **kw):
        """
        List pools.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " lspools"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def cppool(self, **kw):
        """
        Copy content of a pool.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              pool_name(str) : Source pool name.
              dest_pool(str) : Destination pool.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "")
        dest_pool = kw.get("dest_pool", "")
        cmd = self.base_cmd + f" cppool {pool_name} {dest_pool}"
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def purge(self, **kw):
        """Remove all objects from pool <pool_name> without removing it.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              pool_name(str) : Source pool name.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "")
        cmd = self.base_cmd + f" purge {pool_name} --yes-i-really-really-mean-it"
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """
        List objects in pool.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " ls"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def lssnap(self, **kw):
        """
        This method is used to list snaps.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " lssnap"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def mksnap(self, **kw):
        """
        Create snap <snap_name>.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              snap_name(str) : Snap name.

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        snap_name = kw.get("snap_name", "")
        cmd = self.base_cmd + f" mksnap {snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rmsnap(self, **kw):
        """
        Remove snap <snap_name>.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              snap_name(str) : Snap name.

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        snap_name = kw.get("snap_name", "")
        cmd = self.base_cmd + f" rmsnap {snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_(self, **kw):
        """
        Fetch object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.
              outfile(str) : Outfile.

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        outfile = kw.get("outfile", "")
        cmd = self.base_cmd + f" get {obj_name} {outfile}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def put_(self, **kw):
        """
        write object with start offset (default:0).
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.
              infile(str) : Infile.
              offset(str) : Offset(optional)

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        obj_name = kw_copy.pop("obj_name", "")
        infile = kw_copy.pop("infile", "")
        cmd = (
            self.base_cmd
            + f" put {obj_name} {infile}"
            + core_utils.build_cmd_args(kw=kw_copy)
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def append_(self, **kw):
        """
        Append object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.
              infile(str) : Infile.

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        infile = kw.get("infile", "")
        cmd = self.base_cmd + f" append {obj_name} {infile}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def truncate(self, **kw):
        """
        Truncate object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.
              length(str) : Length

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        length = kw.get("length", "")
        cmd = self.base_cmd + f" truncate {obj_name} {length}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def create(self, **kw):
        """
        Create object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" create {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        Remove object(s).
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.
              force-full : Force no matter full or not(optional).

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" rm {obj_name}" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def cp(self, **kw):
        """
        Copy object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.
              target-obj(str) : Target object(optional).

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" cp {obj_name}" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def listxattr(self, **kw):
        """
        List all extended attributes of an object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" listxattr {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def getxattr(self, **kw):
        """
        Dump the extended attribute value of attr of an object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.
              attr(str) : Attribute

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        attr = kw.get("attr", "")
        cmd = self.base_cmd + f" getxattr {obj_name} {attr}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def setxattr(self, **kw):
        """
        Set the value of attr in the extended attributes of an object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.
              attr(str) : Attribute
              value(str) : Value

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        attr = kw.get("attr", "")
        value = kw.get("value", "")
        cmd = self.base_cmd + f" setxattr {obj_name} {attr} {value}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rmxattr(self, **kw):
        """
        Set the value of attr in the extended attributes of an object..
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.
              attr(str) : Attribute

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        attr = kw.get("attr", "")
        cmd = self.base_cmd + f" rmxattr {obj_name} {attr}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def stat(self, **kw):
        """
        Stat the named object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" stat {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def stat2(self, **kw):
        """
        Stat2 the named object (with high precision time).
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" stat2 {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def touch(self, **kw):
        """
        Change the named object modification time.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.
              timestamp(str) : Timestamp(optional).

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" touch {obj_name}" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def mapext(self, **kw):
        """
        Used to map the object name.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str) : Object name.

        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" mapext {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rollback(self, **kw):
        """
        This method is used to roll back object to snap <snap-name>
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str)  : name of the object.
              snap_name(str) : name of the snap.
              base_pool(str): name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        snap_name = kw.get("snap_name", "")
        cmd = self.base_cmd + f" -p {base_pool} rollback {obj_name} {snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def listsnaps(self, **kw):
        """
        This method is used to list the snapshots of this object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              base_pool(str): name of the pool.
              obj_name(str)  : name of the object.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f"-p {base_pool} listsnaps {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def bench(self, **kw):
        """
        This method is used to benchmark for seconds.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              seconds(int): number of seconds.
              mode(str): write|seq|rand.
              concurrent_operations(str): set number of concurrent I/O operations.(default is 16)
              no-cleanup(bool): set this to true to run a write benchmark with the no-cleanup option.(optional)
              run_name(str): used to benchmark a workload test from multiple clients(optional)
                             (default is 'benchmark_last_metadata')
              no_hints(bool): Set this to true to enable the effect of this flag.(optional)
              reuse_bench(bool): Set this to true to enable the effect of this flag.(optional)
              obj_size(int): default object size is 4MB.
              base_pool(str): name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        base_pool = kw_copy.pop("base_pool", "")
        seconds = kw_copy.pop("seconds", "")
        mode = kw_copy.pop("mode", "")
        concurrent_operations = kw_copy.pop("concurrent_operations", 16)
        obj_size = kw.pop("obj_size", "4MB")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = (
            self.base_cmd
            + f" -p {base_pool} bench {seconds} {mode} -b {obj_size} -t {concurrent_operations}"
            + {cmd_args}
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def cleanup(self, **kw):
        """
        This method is used to clean up a previous benchmark operation.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              run_name(str): used to benchmark a workload test from multiple clients
                             (default is 'benchmark_last_metadata')
              prefix(str): prefix
              base_pool(str): name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        run_name = kw.get("run_name", "benchmark_last_metadata")
        base_pool = kw.get("base_pool", "")
        prefix = kw.get("prefix")
        cmd = (
            self.base_cmd
            + f" -p {base_pool} cleanup --run-name {run_name} --prefix {prefix}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def load_gen(self, **kw):
        """
        This method is used to generate load on the cluster.
        Args:
          base_pool(str): name of the pool.
          num_objects(int): number of objects.(optional)
          min_object_size(str): minimum object size.(optional)
          max_object_size(str): maximum object size.(optional)
          max_ops(int): Max number of operations.(optional)
          min_op_len(str) :  Min io size of operations.(optional)
          max_op_len(str) :  Max io size of operations.(optional)
          max_backlog(int): Max backlog size.(optional)
          percent(int): Percent of operations that are read.(optional)
          target_throughput(int): Target throughput (in bytes).(optional)
          run_length(int): Total time (in seconds).(optional)
          offset_align(str): At what boundary to align random op offsets.(optional)

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        base_pool = kw_copy.pop("base_pool", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = self.base_cmd + f" -p {base_pool} load-gen" + cmd_args
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def listomapkeys(self, **kw):
        """
        This method is used to list the keys in the object map.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              base_pool(str): name of the pool.
              obj_name(str): name of the object.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        base_pool = kw.get("base_pool", "")
        cmd = self.base_cmd + f" -p {base_pool} listomapkeys {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def listomapvals(self, **kw):
        """
        This method is used to list the keys and vals in the object map.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              base_pool(str): name of the pool.
              obj_name(str): name of the object.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" -p {base_pool} listomapvals {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def getomapval(self, **kw):
        """
        This method is used to show the value for the specified key in the object's object map.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              base_pool(str): name of the pool.
              obj_name(str): name of the object.
              key(str): object map key.
              file(str): object map key file.
              out-file(bool): if set to false the value will be written to standard output.(optional)

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        base_pool = kw_copy.pop("base_pool", "")
        obj_name = kw_copy.pop("obj_name", "")
        key = kw_copy.pop("key", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = self.base_cmd + f" -p {base_pool} getomapval {obj_name} {key}" + cmd_args
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def setomapval(self, **kw):
        """
        This method is used to set the value of key in the object map of object name.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str): name of the object.
              key(str): object map key.
              file(str): object map key file.
              value(bool): if set to false the value will be written to standard input.(optional)

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        obj_name = kw_copy.pop("obj_name", "")
        key = kw_copy.pop("key", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = self.base_cmd + f" setomapval {obj_name} {key}" + cmd_args
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rmomapkey(self, **kw):
        """
        This method is used to remove key from the object map of object name.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              base_pool(str): name of the pool.
              obj_name(str): name of the object.
              key(str): object map key.
              file(str): object map key file.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        base_pool = kw_copy.pop("base_pool", "")
        obj_name = kw_copy.pop("obj_name", "")
        key = kw_copy.pop("key", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = self.base_cmd + f" -p {base_pool} rmomapkey {obj_name} {key}" + cmd_args
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def clearomap(self, **kw):
        """
        This method is used to clear all the omap keys for the specified objects.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str): name of the object.
              base_pool(str): name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        base_pool = kw_copy.pop("base_pool", "")
        obj_name = kw_copy.pop("obj_name", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = self.base_cmd + f" -p {base_pool} clearomap {obj_name}" + cmd_args
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def getomapheader(self, **kw):
        """
        This method is used to dump the hexadecimal value of the object map header of object name.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str): name of the object.
              base_pool(str): name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f"-p {base_pool} getomapheader {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def setomapheader(self, **kw):
        """
        This method is used to set the value of the object map header of object name.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str): name of the object.
              value(str): value of the object map header of object name.
              base_pool(str): name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        value = kw.get("value", "")
        cmd = self.base_cmd + f" -p {base_pool} setomapheader {obj_name} {value}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def watch(self, **kw):
        """
        This method is used to add watcher on this object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str): name of the object.
              base_pool(str): name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" -p {base_pool} watch {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def notify(self, **kw):
        """
        This method is used to notify watcher of this object with message.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              base_pool(str): name of the pool.
              obj_name(str): name of the object.
              message(str): message with which the watcher must be notified.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        message = kw.get("message", "")
        cmd = self.base_cmd + f" -p {base_pool} notify {obj_name} {message}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def listwatchers(self, **kw):
        """
        This method is used to list the watchers of this object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              base_pool(str): name of the pool.
              obj_name(str): name of the object.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" -p {base_pool} listwatchers {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_alloc_hint(self, **kw):
        """
        This method is used to set allocation hint for an object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              base_pool(str): name of the pool.
              obj_name(str): name of the object.
              expected_object_size(str): expected object size.
              expected_write_size(str): expected write size.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        obj_name = kw.get("obj_name", "")
        base_pool = kw.get("base_pool", "")
        expected_object_size = kw.get("expected_object_size")
        expected_write_size = kw.get("expected_write_size")
        cmd = (
            self.base_cmd
            + f" -p {base_pool} set-alloc-hint {obj_name} {expected_object_size} {expected_write_size}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_redirect(self, **kw):
        """
        This method is used to set redirect target.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              base_pool(str): name of the pool.
              base_obj_name(str): name of the object.
              target_pool(str): target pool.
              target_object(str): name of the target object.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        base_obj_name = kw.get("base_obj_name")
        target_pool = kw.get("target_pool", "")
        target_object = kw.get("target_object", "")
        cmd = (
            self.base_cmd
            + f" -p {base_pool} set-redirect {base_obj_name} --target-pool {target_pool} {target_object}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_chunk(self, **kw):
        """
        This method is used to convert an object to chunked object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              base_pool(str): name of the pool.
              obj_name(str): name of the object.
              source_object(str): name of the source object.
              offset(str): start offset.
              length(str): total time (in seconds).
              target_pool(str): target pool.
              target_object(str): name of the target object.
              target_offset(str): taget offset.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        source_object = kw.get("source_object", "")
        offset = kw.get("offset", "")
        length = kw.get("length")
        target_pool = kw.get("target_pool", "")
        target_object = kw.get("target_object", "")
        target_offset = kw.get("target_offset", "")
        cmd = (
            self.base_cmd
            + f" -p {base_pool} set-chunk {source_object} {offset} {length}"
            + f"--target-pool {target_pool} {target_object} {target_offset}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def tier_promote(self, **kw):
        """
        This method is used to promote the object to the base tier.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str): name of the object.
              base_pool(str): name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" -p {base_pool} tier-promote {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def unset_manifest(self, **kw):
        """
        This method is used to unset redirect or chunked object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str): name of the object.
              base_pool(str): name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" -p {base_pool} unset-manifest {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def tier_flush(self, **kw):
        """
        This method is used to flush the chunked object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str): name of the object.
              base_pool(str): name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" -p {base_pool} tier-flush {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def tier_evict(self, **kw):
        """
        This method is used to evict the chunked object.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              obj_name(str): name of the object.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        base_pool = kw.get("base_pool", "")
        obj_name = kw.get("obj_name", "")
        cmd = self.base_cmd + f" -p {base_pool} tier-evict {obj_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Mds:
    """
    This module provides CLI interface for management of metadata server configuration and administration.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " mds"

    def rm_compat(self, **kw):
        """
        This method is used to remove compatible feature.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example::
            Supported Keys:
              rank(int) : Rank of the MDS daemon.

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        rank = kw.get("rank")
        cmd = self.base_cmd + f" compat rm_compat {rank}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm_incompat(self, **kw):
        """
        This method is used to remove incompatible feature.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example::
            Supported Keys:
              rank(int) : Rank of the MDS daemon.

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        rank = kw.get("rank")
        cmd = self.base_cmd + f" compat rm_incompat {rank}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def compat_show(self, **kw):
        """
        This method is used to show mds compatibility settings.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " compat show"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def fail(self, **kw):
        """
        This method is used to force mds to status fail.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example::
            Supported Keys:
              rank(int) : Rank of the MDS daemon.

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        rank = kw.get("rank")
        cmd = self.base_cmd + f" fail {rank}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        This method is used to remove inactive mds.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example::
            Supported Keys:
              rank(int) : Rank of the MDS daemon.
              name(str) : Name of the file system.

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        name = kw.get("name", "")
        rank = kw.get("rank", "")
        cmd = self.base_cmd + f" rm {name} {rank}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rmfailed(self, **kw):
        """
        This method is used to remove failed mds.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example::
            Supported Keys:
              rank(int) : Rank of the MDS daemon.

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        rank = kw.get("rank")
        cmd = self.base_cmd + f" rmfailed {rank}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def stat(self, **kw):
        """
        This method is used to show MDS status.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example::
            Supported Keys:
              format(str): This is an optional key that helps to prettify and format the JSON data and print it.
        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " stat" + core_utils.build_cmd_args(kw=kw)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def repaired(self, **kw):
        """
        This method is used to mark a damaged MDS rank as no longer damaged.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example::
            Supported Keys:
              rank(int) : Rank of the MDS daemon.

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        rank = kw.get("rank")
        cmd = self.base_cmd + f" repaired {rank}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def allow_new_snaps(self, **kw):
        """
        This method is used to enable snapshots.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " set allow_new_snaps true --yes-i-really-mean-it"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def deactivate(self, **kw):
        """
        This method is used to deactivate the active MDS daemon.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example::
            Supported Keys:
              role(str): Replace <role> with "name of the Ceph File System:rank".

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        role = kw.get("role")
        cmd = self.base_cmd + f" deactivate {role}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def add_data_pool(self, **kw):
        """
        This method is used to add the newly created pool under the control of the metadata servers.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example::
            Supported Keys:
              name(str): name of the newly created pool.

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        name = kw.get("name")
        cmd = self.base_cmd + f" add_data_pool {name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class ErasureCodeProfile:
    """
    Subcommand erasure-code-profile is used for managing the erasure code profiles.
    It uses some additional subcommands.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " erasure_code_profile"

    def get_(self, **kw):
        """
        Subcommand get gets erasure code profile <name>.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                name(str)          :  erasure code profile name

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        name = kw.get("name")
        cmd = f"{self.base_cmd} get {name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """
        Subcommand ls lists all erasure code profiles.

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = f"{self.base_cmd} ls"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        Subcommand rm removes erasure code profile <name>.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                name(str)          :  erasure code profile name

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        name = kw.get("name")
        cmd = f"{self.base_cmd} rm {name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_(self, **kw):
        """
        Subcommand set creates erasure code profile <name> with [<key[=value]> …] pairs.
        Add a –force at the end to override an existing profile (IT IS RISKY).

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                name(str)          :  erasure code profile name
                profile_list(list) :  list of profiles
                force(boolean)     :  True or False

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        name = kw_copy.pop("name", "")
        profile_list = kw_copy.pop("profile_list", "")
        profile_str = " ".join(profile_list)
        cmd = f"{self.base_cmd} set {name} {profile_str}"
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = cmd + cmd_args
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Module:
    """
    This module provides CLI interface to use mgr modules.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " module"

    def ls(self, **kw):
        """
        This method is used to see which modules are available, and which are currently enabled.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              format(str): used to view detailed metadata about disabled modules.(Optional)

        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd_args = core_utils.build_cmd_args(kw=kw)
        cmd = self.base_cmd + " ls" + cmd_args
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def enable(self, **kw):
        """
        This method is used to enable modules.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              module(str): name of the module.

        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        module = kw.get("module")
        cmd = self.base_cmd + " enable" + f" {module}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def disable(self, **kw):
        """
        This method is used to disable modules.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              module(str): name of the module.

        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        module = kw.get("module")
        cmd = self.base_cmd + " disable" + f" {module}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.cli.rbd.config.config import Config
from src.RhcsQeSdk.core.cli.rbd.device import Device
from src.RhcsQeSdk.core.cli.rbd.feature import Feature
from src.RhcsQeSdk.core.cli.rbd.group.group import Group
from src.RhcsQeSdk.core.cli.rbd.image_meta import ImageMeta
from src.RhcsQeSdk.core.cli.rbd.journal import Journal
from src.RhcsQeSdk.core.cli.rbd.lock import Lock
from src.RhcsQeSdk.core.cli.rbd.migration import Migration
from src.RhcsQeSdk.core.cli.rbd.mirror.mirror import Mirror
from src.RhcsQeSdk.core.cli.rbd.namespace import Namespace
from src.RhcsQeSdk.core.cli.rbd.pool import Pool
from src.RhcsQeSdk.core.cli.rbd.snap import Snap
from src.RhcsQeSdk.core.cli.rbd.trash import Trash
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Rbd:
    """
    This module provides CLI interface to manage block device images in a cluster.
    """

    def __init__(self, base_cmd=""):
        self.base_cmd = f"{base_cmd}rbd"
        self.device = Device(self.base_cmd)
        self.mirror = Mirror(self.base_cmd)
        self.trash = Trash(self.base_cmd)
        self.snap = Snap(self.base_cmd)
        self.migration = Migration(self.base_cmd)
        self.pool = Pool(self.base_cmd)
        self.namespace = Namespace(self.base_cmd)
        self.image_meta = ImageMeta(self.base_cmd)
        self.journal = Journal(self.base_cmd)
        self.feature = Feature(self.base_cmd)
        self.config = Config(self.base_cmd)
        self.lock = Lock(self.base_cmd)
        self.group = Group(self.base_cmd)

    def create(self, **kw):
        """
        Creates a block device image.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str) : name of the pool into which image should be stored
                size(int)      : size of image in MegaBytes
                image_name(str):  name of image to be created
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        size = kw.get("size")
        pool_name = kw.get("pool_name")
        cmd = self.base_cmd + f" create {image_name} --size {size} --pool {pool_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """
        Lists block device images within pool.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)  : name of the pool(optional)
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name", "")
        cmd = self.base_cmd + f" ls {pool_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def list_(self, **kw):
        """
        Lists block device images.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            None
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " list"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def info(self, **kw):
        """
        Retrieves information on the block device image.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)  : name of the pool from which info should be retreived(optional)
                image_name(str) :  name of the image from which info should be retreived
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        if kw.get("pool_name"):
            pool_name = kw.get("pool_name")
            cmd = self.base_cmd + f" --image {image_name} -p {pool_name} info"
        else:
            cmd = self.base_cmd + f" --image {image_name} info"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def status(self, **kw):
        """
        Retrieves current state of the block device image operation.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                target_name(str): The operation for which status should be known
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        target = kw.get("target_name")
        cmd = self.base_cmd + f" status {target}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def help(self, **kw):
        """
        Displays help for a particular rbd command and its subcommand.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                command(str): name of command
                sub-command(str): name of sub-command
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        command = kw.get("command")
        sub_command = kw.get("sub-command")
        cmd = self.base_cmd + f" help {command} {sub_command}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def map(self, **kw):
        """
        Maps or unmaps RBD for one or more RBD images.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str): Name of the pool(default is rbd)
                image_name(str): image name
                option(str): options to be passed
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        image_name = kw_copy.pop("image_name", "")
        pool_name = kw_copy.pop("pool_name", "rbd")
        cmd = (
            self.base_cmd
            + f" map {pool_name}/{image_name}"
            + core_utils.build_cmd_args(kw=kw_copy)
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def resize(self, **kw):
        """
        Resize the size of image.
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                image_name(str): name of the image on which resize should happen
                size (int)     :  updated size of image
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        size = kw.get("size")
        cmd = self.base_cmd + f" resize --image {image_name} --size {size}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        Removes the block device image.
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                image_name(str) : name of the image that should be removed
                pool_name(str)  : name of the pool from which image should be retreived(optional)
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        if kw.get("pool_name"):
            pool_name = kw.get("pool_name")
            cmd = self.base_cmd + f" rm {image_name} -p {pool_name}"
        else:
            cmd = self.base_cmd + f" rm {image_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def flatten(self, **kw):
        """
        Flattens the snapshot.
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)      : name of the pool
                image_name(str)     : name of the image
                snap_name(str)      : name of the snapshot
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        cmd = self.base_cmd + f" flatten {pool_name}/{image_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def clone(self, **kw):
        """
        Clones the snapshot.
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)         : name of the pool
                parent_image_name(str) : name of the parent image
                child_image_name(str)  : name of the child image
                snap_namestr)          : name of the snapshot
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        parent_image_name = kw.get("parent_image_name")
        child_image_name = kw.get("child_image_name")
        pool_name = kw.get("pool_name")
        snap_name = kw.get("snap_name")
        cmd = (
            self.base_cmd
            + f" clone {pool_name}/{parent_image_name}@{snap_name} {pool_name}/{child_image_name}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def children(self, **kw):
        """
        Lists the children of a snapshot.
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str)  : name of the pool
                image_name(str) : name of the image
                snap_name(str)  : name of the snapshot
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        image_name = kw.get("image_name")
        pool_name = kw.get("pool_name")
        snap_name = kw.get("snap_name")
        cmd = self.base_cmd + f" children {pool_name}/{image_name}@{snap_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def bench(self, **kw):
        """
        This method is used to generate a series of IOs to the image and measure the IO throughput and latency.
        Args:
          kw(dict): Key/value pairs that needs to be provided to the installer
          Example:
            Supported keys:
              io-type(str): can be read | write | readwrite | rw.
              image_name(str): name of the image.
              pool_name(str): name of the pool.
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        image_name = kw_copy.pop("image_name")
        pool_name = kw_copy.pop("pool_name")
        io_type = kw_copy.pop("io-type")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = (
            self.base_cmd
            + f" bench {image_name}/{pool_name} --io-type {io_type}"
            + cmd_args
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.cli.ceph.blocklist import Blocklist
from src.RhcsQeSdk.core.cli.ceph.crush import Crush
from src.RhcsQeSdk.core.cli.ceph.erasurecodeprofile import ErasureCodeProfile
from src.RhcsQeSdk.core.cli.ceph.pool import Pool
from src.RhcsQeSdk.core.cli.ceph.tier import Tier
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Osd:
    """This module provides CLI interface to manage OSD configuration and administration via ceph osd."""

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " osd"
        self.pool = Pool(self.base_cmd)
        self.crush = Crush(self.base_cmd)
        self.blocklist = Blocklist(self.base_cmd)
        self.erasure_code_profile = ErasureCodeProfile(self.base_cmd)
        self.tier = Tier(self.base_cmd)

    def blocked_by(self, **kw):
        """
        Subcommand blocked-by prints a histogram of which OSDs are blocking their peers

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = f"{self.base_cmd} blocked-by"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def create(self, **kw):
        """
        Create the OSD. If no UUID is given, it will be set automatically when the OSD starts up.
        The following command will output the OSD number, which you will need for subsequent steps.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              UUID(str) : new OSD will have the specified uuid (optional)
              ID(str)   : auth entity client.osd.<id> (optional)
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        UUID = kw.get("UUID")
        ID = kw.get("ID")
        cmd = f"{self.base_cmd} create"
        if UUID:
            cmd = cmd + f" {UUID}"
        if ID:
            cmd = cmd + f" {ID}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def new(self, **kw):
        """
        Subcommand new can be used to create a new OSD or to recreate a previously destroyed OSD with a specific id.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              UUID(str)        : new OSD will have the specified uuid
              ID(str)          : auth entity client.osd.<id>
              params_file(str) : JSON file containing the base64 cephx key for auth entity client.osd.<id>,
                                 as well as optional base64 cepx key for dm-crypt lockbox access
                                 and a dm-crypt key (optional)

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        UUID = kw.get("UUID", "")
        ID = kw.get("ID", "")
        params_file = kw.get("params_file")
        cmd = f"{self.base_cmd} new {UUID} {ID} -i {params_file}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def deep_scrub(self, **kw):
        """
        Subcommand deep-scrub initiates deep scrub on specified osd.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              who(str)  :  entity

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        who = kw.get("who")
        cmd = f"{self.base_cmd} deep-scrub {who}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def df(self, **kw):
        """
        Subcommand df shows OSD utilization

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              option(str)  :  values are plain/tree

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        option = kw.get("option")
        cmd = f"{self.base_cmd} df {option}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def down(self, **kw):
        """
        Subcommand down sets osd(s) <id> [<id>…] down.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              id_list(list)  :  list of osd id's

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        id_list = kw.get("id_list")
        id_str = " ".join(id_list)
        cmd = f"{self.base_cmd} down {id_str}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def dump(self, **kw):
        """
        Subcommand dump prints summary of OSD map.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)  :  OSD id

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        cmd = f"{self.base_cmd} dump {osd_id}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def find(self, **kw):
        """
        Subcommand find finds osd <id> in the CRUSH map and shows its location.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)  :  OSD id

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        cmd = f"{self.base_cmd} find {osd_id}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def getcrushmap(self, **kw):
        """
        Subcommand getcrushmap gets CRUSH map.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)  :  OSD id

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        cmd = f"{self.base_cmd} getcrushmap {osd_id}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def getmap(self, **kw):
        """
        Subcommand getmap gets OSD map.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              epoch(int): epoch
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("epoch", "")
        cmd = f"{self.base_cmd} getmap {osd_id}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def getmaxosd(self, **kw):
        """
        Subcommand getmaxosd shows largest OSD id.

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = f"{self.base_cmd} getmaxosd"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def in_(self, **kw):
        """
        Subcommand in sets osd(s) <id> [<id>…] in.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              id_list(list)  :  list of osd id's

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        id_list = kw.get("id_list")
        id_str = " ".join(id_list)
        cmd = f"{self.base_cmd} in {id_str}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def lost(self, **kw):
        """
        Subcommand lost marks osd as permanently lost. THIS DESTROYS DATA IF NO MORE REPLICAS EXIST, BE CAREFUL.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)  :  OSD id

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        cmd = f"{self.base_cmd} lost {osd_id} --yes-i-really-mean-it"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ls(self, **kw):
        """
        Subcommand ls shows all OSD ids.

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = f"{self.base_cmd} ls"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def lspools(self, **kw):
        """
        The lspools command will display the list of all pools in the cluster.

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " lspools"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def map_(self, **kw):
        """
        Subcommand map finds pg for <object> in <pool>.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              pool_name(str)  :  pool name
              object_name(str)  :  object name

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name")
        object_name = kw.get("object_name")
        cmd = f"{self.base_cmd} map {pool_name} {object_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def metadata(self, **kw):
        """
        Subcommand metadata fetches metadata for osd <id>.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)  :  osd id

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        cmd = f"{self.base_cmd} metadata {osd_id} (default all)"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def out(self, **kw):
        """
        Subcommand out sets osd(s) <id> [<id>…] out.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              id_list(list)  :  list of osd id's

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        id_list = kw.get("id_list")
        id_str = " ".join(id_list)
        cmd = f"{self.base_cmd} out {id_str}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def ok_to_stop(self, **kw):
        """
        Subcommand ok-to-stop checks whether the list of OSD(s) can be stopped
        without immediately making data unavailable.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              id_list(str)  :  list of osd id's
              max(int)      :  maximum number up to <num> OSDs IDs will return (including the provided OSDs)
                               that can all be stopped simultaneously (optional)

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        id_list = kw_copy.pop("id_list", "")
        id_str = " ".join(id_list)
        cmd = f"{self.base_cmd} ok-to-stop {id_str}"
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = cmd + cmd_args
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def pause(self, **kw):
        """
        Subcommand pause pauses osd.

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = f"{self.base_cmd} pause"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def perf(self, **kw):
        """
        Subcommand perf prints dump of OSD perf summary stats.

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        cmd = f"{self.base_cmd} perf"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def pg_temp(self, **kw):
        """
        Subcommand pg-temp set pg_temp mapping pgid:[<id> [<id>…]] (developers only).

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              pgid(str)        :  id of pg
              id_list(list)  :  list of id's

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pgid = kw.get("pgid")
        id_list = kw.get("id_list")
        id_str = " ".join(id_list)
        cmd = f"{self.base_cmd} pg-temp {pgid} {id_str}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def force_create_pg(self, **kw):
        """
        Subcommand force-create-pg forces creation of pg <pgid>.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              pgid(str)  :  id of pg

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pgid = kw.get("pgid")
        cmd = f"{self.base_cmd} force-create-pg {pgid}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def primary_affinity(self, **kw):
        """
        Subcommand primary-affinity adjust osd primary-affinity from 0.0 <=<weight> <= 1.0

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_name(str)  :  name of osd
              osd_id(str)    :  osd id (alternate to osd_name)
              weight(float)  :  0.0 < weight < 1.0

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_name = kw.get("osd_name")
        osd_id = kw.get("osd_id")
        weight = kw.get("weight")
        cmd = f"{self.base_cmd} primary-affinity {osd_name or osd_id} {weight}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def primary_temp(self, **kw):
        """
        Subcommand primary-temp sets primary_temp mapping pgid:<id>|-1 (developers only).

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              pgid(str)  :  id of pg
              osd_id(str)  :  id

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pgid = kw.get("pgid")
        osd_id = kw.get("osd_id")
        cmd = f"{self.base_cmd} primary-temp {pgid} {osd_id}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def repair(self, **kw):
        """
        Subcommand repair initiates repair on a specified osd.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              who(str)  :  osd

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        who = kw.get("who")
        cmd = f"{self.base_cmd} repair {who}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def reweight(self, **kw):
        """
        Subcommand reweight reweights osd to 0.0 < <weight> < 1.0.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)  :  id of osd
              weight(float)  :  0.0 < weight < 1.0

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        weight = kw.get("weight")
        cmd = f"{self.base_cmd} reweight {osd_id} {weight}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def reweight_by_pg(self, **kw):
        """
        Subcommand reweight-by-pg reweight OSDs by PG distribution [overload-percentage-for-consideration, default 120].

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)   :  osd id
              poolname_list(str)  :  list of pool names

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        poolname_list = kw.get("poolname_list")
        poolname_str = " ".join(poolname_list)
        cmd = f"{self.base_cmd} reweight-by-pg {osd_id} {poolname_str} --no-increasing"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def reweight_by_utilization(self, **kw):
        """
        Subcommand reweight-by-utilization reweights OSDs by utilization.
        It only reweights outlier OSDs whose utilization exceeds the average

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)   :  osd id
              overload_threshold(int)  :  default 120
              max_weight_change(float) :  default 0.05
              max_osds_to_adjust(int)  :  default 4

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        overload_threshold = kw.get("overload_threshold", 120)
        max_weight_change = kw.get("max_weight_change", 0.05)
        max_osds_to_adjust = kw.get("max_osds_to_adjust", 4)
        cmd = (
            f"{self.base_cmd} reweight-by-utilization {osd_id}"
            f" {overload_threshold} {max_weight_change} {max_osds_to_adjust} --no-increasing"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        Subcommand rm removes osd(s) <id> [<id>…] from the OSD map.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              id_list(list)  :  list of osd id's

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        id_list = kw.get("id_list")
        id_str = " ".join(id_list)
        cmd = f"{self.base_cmd} rm {id_str}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def destroy(self, **kw):
        """
        Subcommand destroy marks OSD id as destroyed, removing its cephx entity’s keys
        and all of its dm-crypt and daemon-private config key entries.
        This command will not remove the OSD from crush, nor will it remove the OSD from the OSD map.
        Instead, once the command successfully completes, the OSD will show marked as destroyed.
        In order to mark an OSD as destroyed, the OSD must first be marked as lost.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)  :  osd id

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        cmd = f"{self.base_cmd} destroy {osd_id} --yes-i-really-mean-it"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def purge(self, **kw):
        """
        Subcommand purge performs a combination of osd destroy, osd rm and osd crush remove.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)  :  osd id

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        cmd = f"{self.base_cmd} purge {osd_id} --yes-i-really-mean-it"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def safe_to_destroy(self, **kw):
        """
        Subcommand safe-to-destroy checks whether it is safe to remove or destroy an OSD
        without reducing overall data redundancy or durability.
        It will return a success code if it is definitely safe, or an error code and informative message if it is not or
        if no conclusion can be drawn at the current time.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              id_list(list)  :  list of osd id's

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        id_list = kw.get("id_list")
        id_str = " ".join(id_list)
        cmd = f"{self.base_cmd} safe-to-destory {id_str}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def scrub(self, **kw):
        """
        Subcommand scrub initiates scrub on specified osd.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              who(str)  :  entity

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        who = kw.get("who")
        cmd = f"{self.base_cmd} scrub {who}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_(self, **kw):
        """
        Subcommand set sets cluster-wide <flag> by updating OSD map.
        The full flag is not honored anymore since the Mimic release,
        and ceph osd set full is not supported in the Octopus release.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              option(str)  :  pause|noup|nodown|noout|noin|nobackfill|norebalance|
                              norecover|noscrub|nodeep-scrub|notieragent

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        option = kw.get("option")
        cmd = f"{self.base_cmd} set {option}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def setcrushmap(self, **kw):
        """
        Subcommand setcrushmap sets crush map from input file.

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = f"{self.base_cmd} setcrushmap"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def setmaxosd(self, **kw):
        """
        Subcommand setmaxosd sets new maximum osd value.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              max_osd_val(int)  :  new maximum osd value.

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        max_osd_val = kw.get("max_osd_val")
        cmd = f"{self.base_cmd} setmaxosd {max_osd_val}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_require_min_compat_client(self, **kw):
        """
        Subcommand set-require-min-compat-client enforces the cluster to be backward compatible
        with the specified client version

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              version(str)  :  version of the client

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        version = kw.get("version")
        cmd = f"{self.base_cmd} set-require-min-compat-client {version}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def stat(self, **kw):
        """
        Subcommand stat prints summary of OSD map.

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = f"{self.base_cmd} stat"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def tree(self, **kw):
        """
        Subcommand tree prints OSD tree.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              osd_id(str)  :  osd_id

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        osd_id = kw.get("osd_id")
        cmd = f"{self.base_cmd} tree {osd_id}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def unpause(self, **kw):
        """
        Subcommand unpause unpauses osd.

        Args:
            None

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = f"{self.base_cmd} unpause"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def unset(self, **kw):
        """
        Subcommand unset unsets cluster-wide <flag> by updating OSD map.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              option(str)  :  pause|noup|nodown|noout|noin|nobackfill|
                              norebalance|norecover|noscrub|nodeep-scrub|notieragent

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        option = kw.get("option")
        cmd = f"{self.base_cmd} unset {option}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class RolePolicy:
    """
    This module manages role policies
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " role-policy"

    def put(self, **kw):
        """
        Add/update permission policy to role.

        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              role-name(str): Name of the role.
              policy-name(str): Name of the policy.
              policy-doc(str) : The Permission policy document.
              infile(str)     : passing policy-doc as a file. Alternate to policy_doc

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        role_name = kw_copy.pop("role-name", "")
        policy_name = kw_copy.pop("policy-name", "")
        cmd = f"{self.base_cmd} put --role-name={role_name} --policy-name={policy_name}"
        cmd = cmd + core_utils.build_cmd_args(kw=kw_copy)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def list_(self, **kw):
        """
        List the policies attached to a role.

        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              role-name(str): Name of the role.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        role_name = kw.get("role-name")
        cmd = f"{self.base_cmd} list --role-name={role_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def get_(self, **kw):
        """
        Get the specified inline policy document embedded with the given role.

        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              role-name(str): Name of the role.
              policy-name(str): Name of the policy.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        role_name = kw.get("role-name")
        policy_name = kw.get("policy-name")
        cmd = f"{self.base_cmd} get --role-name={role_name} --policy-name={policy_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def delete(self, **kw):
        """
        Remove the policy attached to a role

        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              role-name(str): Name of the role.
              policy-name(str): Name of the policy.

        Returns:
          Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        role_name = kw.get("role-name")
        policy_name = kw.get("policy-name")
        cmd = f"{self.base_cmd} delete --role-name={role_name} --policy-name={policy_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import src.RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Key:
    """
    This module provides CLI interface to implement radosgw-admin user key commands.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " key"

    def create(self, **kw):
        """
        The command create will create a user key.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

            user_id(str): user id of a particular user.
            key_type(str): key_type of the user.(s3 for user and swift for subuser)

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run
        """
        kw = kw.get("kw")
        user_id = kw.get("user_id", "")
        key_type = kw.get("key_type", "")
        cmd = self.base_cmd + " create "
        if key_type == "s3":
            cmd = (
                cmd
                + f"--uid={user_id} --key-type={key_type} --gen-access-key --gen-secret"
            )
        else:
            cmd = (
                cmd
                + f"--subuser={user_id}:swift --key-type={key_type} --gen-access-key --gen-secret"
            )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        The command rm will remove a user key.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:

            user_id(str): user id of a particular user.
            access_key(str): the access key for a specific user.

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run
        """
        kw = kw.get("kw")
        user_id = kw.get("user_id", "")
        access_key = kw.get("access_key", "")
        cmd = self.base_cmd + f" rm --uid={user_id} --access-key {access_key}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import src.RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Daemon:
    """
    This module provides a command line interface (CLI) to ceph orch daemon modules.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " daemon"

    def add_(self, **kw):
        """
        Deploys daemons based on provided placement specification.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service_name(str): name of service.
              placement(str): name of host to be deployed into.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        service_name = kw.get("service_name")
        placement = kw.get("placement")
        cmd = self.base_cmd + f" add {service_name} --placement={placement}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        Removes a daemon from host.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service_name(str): name of service.
              placement(str): name of host to be removed from.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        service_name = kw.get("service_name")
        placement = kw.get("placement")
        cmd = self.base_cmd + f" rm {service_name} --placement={placement}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def restart(self, **kw):
        """
        Restarts a daemon on host.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
              service_name(str): name of service.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        service_name = kw.get("service_name")
        cmd = self.base_cmd + f" restart {service_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Tier:
    """Subcommand tier is used for managing tiers. It uses some additional subcommands."""

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " tier"

    def add_(self, **kw):
        """
        Subcommand add adds the tier <tierpool> (the second one) to base pool <pool> (the first one).

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                poolnames(list)           : list of pool names.two poolnames actually.
                                            the second is tierpool and the first is pool
                force-nonempty(boolean)   : True or False

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        pool_names = kw_copy.pop("poolnames", "")
        cmd = f"{self.base_cmd} add {pool_names[0]} {pool_names[1]}"
        cmd = cmd + core_utils.build_cmd_args(kw=kw_copy)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def add_cache(self, **kw):
        """
        Subcommand add-cache adds a cache <tierpool> (the second one) of size <size>
        to existing pool <pool> (the first one).

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
                poolnames(list)           : list of pool names.two poolnames actually.
                                            the second is tierpool and the first is pool
                size(int)        : cache size

        Returns:
            Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_names = kw.get("poolnames")
        size = kw.get("size")
        cmd = f"{self.base_cmd} add-cache {pool_names[0]} {pool_names[1]} {size}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def cache_mode(self, **kw):
        """
        Subcommand cache-mode specifies the caching mode for cache tier <pool>.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              poolname(str) : name of the pool.
              option(str)    : writeback|readproxy|readonly|none
        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("poolname")
        option = kw.get("option")
        cmd = f"{self.base_cmd} cache-mode {pool_name} {option}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def remove_(self, **kw):
        """
        Subcommand remove removes the tier <tierpool> (the second one) from base pool <pool> (the first one).

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              poolnames(list) :  list of pool names.two poolnames actually.
                                 the second is tierpool and the first is pool

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_names = kw.get("poolnames")
        cmd = f"{self.base_cmd} remove {pool_names[0]} {pool_names[1]}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def remove_overlay(self, **kw):
        """
        Subcommand remove-overlay removes the overlay pool for base pool <pool>.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              poolname(str) : name of the pool.

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("poolname")
        cmd = f"{self.base_cmd} remove-overlay {pool_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def set_overlay(self, **kw):
        """
        Subcommand set-overlay set the overlay pool for base pool <pool> to be <overlaypool>.

        Args:
            kw(Dict): Key/value pairs that needs to be provided to the installer.
            Example::
            Supported Keys:
              poolnames(list) :  list of pool names.two poolnames actually.
                                 the second is tierpool and the first is pool

        Returns:
            Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_names = kw.get("poolnames")
        cmd = f"{self.base_cmd} set-overlay {pool_names[0]} {pool_names[1]}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



def _get_module(
    obj,
    module,
    service,
):
    """
    Module to coordinate logic to classes and trigger methods based on user input
    provided in suite file using DFS.
    Args:
        obj(object)   :  object of a class
        module(str)   :  name of the module excluding .py
        service(str)  :  method name

    Returns:
        method() : the path of the file through which we can execute the sdk layer
    """
    if type(obj).__module__ == "builtins":
        return None
    if str(obj.__class__.__name__).lower().replace("_", "") == module.lower():
        method = getattr(obj, service, None)
        if method:
            return method
        return None

    else:
        for atrb, val in obj.__dict__.items():
            if not atrb.startswith("__"):
                obj_atrb = getattr(obj, atrb)
                method = _get_module(obj_atrb, module, service)
                if method:
                    return method
        return None


def get_module(**kw):
    obj = kw.get("obj")
    component = kw.get("component")
    module = kw.get("module")
    service = kw.get("service")
    obj = getattr(obj, component)
    return _get_module(obj, module, service)


# kw = {
#     "module":"cephadm",
#     "service":"bootstrap",
#     "component":"cephadm",
#     "obj": CLI()
# }

# m = get_module(kw=kw)
# print(m)
# m()



import logging

import src.RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Metadata:
    """
    This module provides CLI interface for commands via radosgw-admin metadata.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " metadata"

    def get_(self, **kw):
        """Get metadata info.
        Args:
                bucket(str) :  Holds a mapping between bucket name and bucket instance id.
                bucket.instance(str) : Holds bucket instance information.
                user(str) : Holds user information.
                key(str) : Holds Metadata key
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        cmd = self.base_cmd + " get"
        kw = kw.get("kw")
        bucket = kw.get("bucket", "")
        bucket_instance = kw.get("bucket.instance", "")
        user = kw.get("user", "")
        metadata_key = kw.get("key", "")
        if bucket:
            cmd = cmd + f" bucket:{bucket}"
        elif bucket_instance:
            cmd = cmd + f" bucket.instance:{bucket_instance}"
        elif user:
            cmd = cmd + f" user:{user}"
        elif metadata_key:
            cmd = cmd + f" --metadata-key={metadata_key}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def put_(self, **kw):
        """Put metadata info.
        Args:
                user-id(str) : User ID
                bucket-id(str) : Bucket ID
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        cmd = self.base_cmd + " put"
        kw = kw.get("kw")
        user_id = kw.get("user-id", "")
        bucket_id = kw.get("bucket-id", "")
        if user_id:
            cmd = cmd + f" user:{user_id}"
        elif bucket_id:
            cmd = cmd + f" bucket:{bucket_id}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """Remove metadata info.
        Args:
                bucket(str) :  Holds a mapping between bucket name and bucket instance id.
                bucket.instance(str) : Holds bucket instance information.
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        cmd = self.base_cmd + " rm"
        kw = kw.get("kw")
        bucket = kw.get("bucket", "")
        bucket_instance = kw.get("bucket.instance", "")
        if bucket:
            cmd = cmd + f" bucket:{bucket}"
        elif bucket_instance:
            cmd = cmd + f" bucket.instance:{bucket_instance}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def list_(self, **kw):
        """List metadata info.
        Args:
                option(str) : bucket | bucket.instance | user
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        option = kw.get("option", "")
        cmd = self.base_cmd + " list" + f" {option}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import src.RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Pool:
    """
    This module provides CLI interface to manage pools in rbd via rbd pool command.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " pool"

    def init(self, **kw):
        """
        Initiates rbd application on specified pool.
        Args:
        kw(dict): Key/value pairs that needs to be provided to the installer
            Example::
            Supported keys:
                pool_name(str) : name of the pool
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        pool_name = kw.get("pool_name")
        cmd = self.base_cmd + f" init -p {pool_name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Device:
    """
    This Class provides wrappers for rbd device commands.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " device"

    def attach(self, **kw):
        """Wrapper for rbd device attach.

        Attach image to device.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                device-type (str): device type [ggate, krbd (default), nbd]
                pool(str): pool name
                namespace(str): namespace name
                image(str): image name
                snap(str): snapshot name
                device(str): specify device path
                show-cookie(bool): show device cookie
                cookie(str): specify device cookie
                read-only(bool): attach read-only
                force(bool): force attach
                exclusive(bool): disable automatic exclusive lock transitions
                quiesce(bool): use quiesce hooks
                quiesce-hook(str): quiesce hook path
                options(str): device specific options
                image_or_snap_spec(str): [<pool-name>/[<namespace>/]]<image-name>[@<snap-name>]

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """

        kw_copy = deepcopy(**kw)
        image_or_snap_spec = kw_copy.pop("image_or_snap_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} attach {image_or_snap_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def detach(self, **kw):
        """Wrapper for rbd device detach.

        Detach image from device.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                device-type (str): device type [ggate, krbd (default), nbd]
                pool(str): pool name
                namespace(str): namespace name
                image(str): image name
                snap(str): snapshot name
                options(str): device specific options
                image_snap_or_device_spec: [<pool-name>/]<image-name>[@<snap-name>]
                                            or <device-path>
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """

        kw_copy = deepcopy(**kw)
        image_snap_or_device_spec = kw_copy.pop("image_snap_or_device_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} detach {image_snap_or_device_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def list_(self, **kw):
        """Wrapper for rbd device list.

        List mapped rbd images.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                device-type (str): device type [ggate, krbd (default), nbd]
                format(str): output format (plain, json, or xml) [default: plain]
                pretty-format(bool): True
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """

        cmd_args = core_utils.build_cmd_args(kw=kw)

        cmd = f"{self.base_cmd} list{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def map(self, **kw):
        """Wrapper for rbd device map.

        Map an image to a block device.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                device-type (str): device type [ggate, krbd (default), nbd]
                pool(str): pool name
                namespace(str): namespace name
                image(str): image name
                snap(str): snapshot name
                device(str): specify device path
                show-cookie(bool): show device cookie
                cookie(str): specify device cookie
                read-only(bool): attach read-only
                force(bool): force attach
                exclusive(bool): disable automatic exclusive lock transitions
                quiesce(bool): use quiesce hooks
                quiesce-hook(str): quiesce hook path
                options(str): device specific options
                image_or_snap_spec(str): [<pool-name>/[<namespace>/]]<image-name>[@<snap-name>]
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """

        kw_copy = deepcopy(**kw)
        image_or_snap_spec = kw_copy.pop("image_or_snap_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} map {image_or_snap_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def unmap(self, **kw):
        """Wrapper for rbd device unmap.

        Unmap a rbd device.
        Args:
        kw: Key value pair of method arguments
            Example::
            Supported keys:
                device-type (str): device type [ggate, krbd (default), nbd]
                pool(str): pool name
                namespace(str): namespace name
                image(str): image name
                snap(str): snapshot name
                options(str): device specific options
                image_snap_or_device_spec: [<pool-name>/]<image-name>[@<snap-name>]
                                            or <device-path>
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """

        kw_copy = deepcopy(**kw)
        image_snap_or_device_spec = kw_copy.pop("image_snap_or_device_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} unmap {image_snap_or_device_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging
from copy import deepcopy

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Migration:

    """
    This Class provides wrappers for rbd migration commands.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " migration"

    def abort(self, **kw):
        """Wrapper for rbd migration abort.

        Cancel interrupted image migration.

        Args:
            kw: Key value pair of method arguments
        Example::
        Supported keys:
            pool(str): Name of the pool of which peer is to be bootstrapped.
            namespace(str): name of the namespace
            image(str): image name
            no-progress(bool): True - disable progress output
            image_spec(str): image specification

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        image_spec = kw_copy.pop("image_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} abort {image_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def commit(self, **kw):
        """Wrapper for rbd migration commit.

        Commit image migration.

        Args:
            kw: Key value pair of method arguments
        Example::
        Supported keys:
            pool(str): Name of the pool of which peer is to be bootstrapped.
            namespace(str): name of the namespace
            image(str): image name
            no-progress(bool): True - disable progress output
            force(bool): True - proceed even if the image has children
            image_spec(str): image specification

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        image_spec = kw_copy.pop("image_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} commit {image_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def execute(self, **kw):
        """Wrapper for rbd migration abort.

        Execute image migration.

        Args:
            kw: Key value pair of method arguments
        Example::
        Supported keys:
            pool(str): Name of the pool of which peer is to be bootstrapped.
            namespace(str): name of the namespace
            image(str): image name
            no-progress(bool): True - disable progress output
            image_spec(str): image specification

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        image_spec = kw_copy.pop("image_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} execute {image_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def prepare(self, **kw):
        """Wrapper for rbd migration abort.

        Prepare image migration.

        Args:
            kw: Key value pair of method arguments
        Example::
        Supported keys:
            import-only(bool): only import data from source
            source-spec-path(str): source-spec file
            source-spec(str): source-spec
            pool(str): name of the pool
            namespace(str): source namespace name
            image(str): source image name
            snap(str):  source snapshot name
            dest-pool(str): destination pool name
            dest-namespace(str):  destination namespace name
            dest(str):  destination image name
            image-format(str):  image format [default: 2]
            object-size(str): object size in B/K/M [4K <= object size <= 32M]
            image-feature(str): image features
            image-shared(bool): shared image
            stripe-unit(str): stripe unit in B/K/M
            stripe-count(str):  stripe count
            data-pool(str): data pool
            mirror-image-mode(str): mirror image mode [journal or snapshot]
            journal-splay-width(str): number of active journal objects
            journal-object-size(str): size of journal objects [4K <= size <= 64M]
            journal-pool(str):  pool for journal objects
            flatten(bool): True - fill clone with parent data (make it independent)
            source-image-or-snap-spec(str): source image or snapshot specification
            dest-image-spec: destination image specification
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        source_spec = kw_copy.pop("source-image-or-snap-spec", "")
        destination_spec = kw_copy.pop("dest-image-spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} prepare {source_spec} {destination_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



from src.RhcsQeSdk.core.cli.rbd.mirror.schedule import Schedule


class Snapshot:
    """
    This module provides CLI interface to manage the mirror snapshots.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " snapshot"
        self.schedule = Schedule(self.base_cmd)



import logging

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Subuser:
    """
    This module provides CLI interface for commands used for RGW subuser management.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " subuser"

    def create(self, **kw):
        """Create a new subuser (primarily useful for clients using the Swift API).
        Args:
                uid(str) : User ID
                access(str) : [ read | write | readwrite | full ]
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        uid = kw.get("uid", "")
        cmd = (
            self.base_cmd
            + " create"
            + f" --uid={uid} --subuser={uid} "
            + core_utils.build_cmd_args(kw=kw)
        )
        logger.info(f"Running commmand {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def modify(self, **kw):
        """Modify existing subuser.
        Args:
                uid(str) : User ID
                access(str) : [ read | write | readwrite | full ]
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        uid = kw.get("uid", "")
        cmd = (
            self.base_cmd
            + " modify"
            + f" --uid={uid} --subuser={uid} "
            + core_utils.build_cmd_args(kw=kw)
        )
        logger.info(f"Running commmand {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """Remove a new subuser.
        Args:
                uid(str) : User ID
        Returns:
            Dict(str)
            A mapping of host strings to the given task’s return value for that host’s execution run
        """
        kw = kw.get("kw")
        uid = kw.get("uid", "")
        cmd = self.base_cmd + " rm" + f" --subuser={uid}"
        logger.info(f"Running commmand {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



import logging

import src.RhcsQeSdk.core.cli.fabfile as fabfile

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Mon:
    """
    This module provides CLI interface for the management of monitor configuration and administration.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " mon"

    def add_(self, **kw):
        """
        This method is used to add new monitor named with a specified name at a specified address.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported Keys:
              name(str) : Name of the monitor.
              addr(str) : IP Address (Host) on which Monitor daemon to be deployed.

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        name = kw.get("name")
        addr = kw.get("addr")
        cmd = self.base_cmd + f" add {name} {addr}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def dump(self, **kw):
        """
        This method is used to dump the formatted monmap.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.

        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " dump"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def getmap(self, **kw):
        """
        This method is used to get the monmap.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported Keys:
              MONMAP(str): This key is used to store the path of monmap.

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        monmap = kw.get("MONMAP")
        cmd = self.base_cmd + f" getmap -o {monmap}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def remove_(self, **kw):
        """
        This method is used to remove monitor.
        Args:
          kw(Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported Keys:
              name(str) : Name of the monitor.

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        name = kw.get("name")
        cmd = self.base_cmd + f" remove {name}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def stat(self, **kw):
        """
        This method is used to summarize the monitor status.
        Args:
          None

        Returns:
          Dict(str)
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " stat"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



class RedefineArgs:
    def __init__(self):
        pass

    def get_args(self, **kw):
        kw = kw.get("kw")
        step = kw.get("step")
        cls = step.get("class")
        method = step.get("method")
        component = step.get("component")
        redefine_method_name = f"{component}_{cls}_{method}"
        args = step.get("args")
        redefine_method = getattr(self, redefine_method_name)
        if redefine_method:
            args = redefine_method(kw=kw)
        return args

    def cephadm_cephadm_bootstrap(self, **kw):
        kw = kw.get("kw")
        step = kw.get("step")
        cluster_name = kw.get("cluster_name")
        ceph_cluster_dict = kw.get("ceph_cluster_dict")
        args = step.get("args")
        role = step.get("role")
        if len(role.split(":")) == 2:
            cluster_and_role = role.split(":")
            cluster_name = cluster_and_role[0]
            role = cluster_and_role[1]
        if args.get("registry-url"):
            args["registry-username"] = "cephuser"
            args["registry-password"] = "cephuser"
        if args.get("mon-ip"):
            ips = [
                ceph_cluster_dict[cluster_name]
                .get_node_by_hostname(args.get("mon-ip"))
                .ip_address
            ]
        else:
            ips = []
            nodes = ceph_cluster_dict[cluster_name].get_nodes("mon")
            for node in nodes:
                ips.append(node.ip_address)
        args["mon-ip"] = " ".join(ips)
        return args

    def ceph_orch_apply(self, **kw):
        kw = kw.get("kw")
        step = kw.get("step")
        args = step.get("args")
        role = step.get("role")
        service_name = args.get("service_name")
        if len(role.split(":")) == 2:
            cluster_and_role = role.split(":")
            # cluster_name = cluster_and_role[0]
            role = cluster_and_role[1]
        placement = args.get("placement", {})
        if placement:
            if "label" in placement:
                args["label"] = placement["label"]
            if "nodes" in placement:
                sep = placement.get("sep", " ")
                nodes = placement.get("nodes", "")
                nodes_str = f"{sep}".join(nodes)
                count = placement.get("count", "")
                if service_name == "rgw" and placement.get("count-per-host", ""):
                    count = placement.get("count-per-host", None)
                nodes_str = f"{count}{sep}{nodes_str}"
                args["nodes_str"] = nodes_str
        return args



import logging
from copy import deepcopy

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.cli.ceph.orch.label import Label
from src.RhcsQeSdk.core.cli.ceph.orch.maintanence import Maintanence
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Host:
    """
    This module provides a command line interface (CLI) to ceph orch host.
    """

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " host"
        self.label = Label(self.base_cmd)
        self.maintanence = Maintanence(self.base_cmd)

    def ls(self, **kw):
        """
        Displays the current hosts and labels.
        Args:
            None
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cmd = self.base_cmd + " ls"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def add_(self, **kw):
        """
        To add new hosts to cluster.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
                host_name(str): name of host.
                labels(str): name of label.(Optional).
                ip_address(str): Ipaddress of host.

        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        cluster_name = kw.get("cluster_name")
        kw_copy = deepcopy(kw)
        host_name = kw_copy.pop("host_name")
        ip_address = kw_copy.pop("ip_address", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        if host_name == "all":
            nodes = core_utils.parse_host_name(host_name, cluster_name)
            for node in nodes:
                cmd = (
                    self.base_cmd
                    + f" add {node.get('name')} {node.get('node_obj').private_ips[0]}"
                    + cmd_args
                )
                logger.info(f"Running command {cmd}")
                fabfile.run_command(cmd, config=kw.get("env_config"))
        else:
            cmd = self.base_cmd + f" add {host_name} {ip_address}" + cmd_args
            logger.info(f"Running command {cmd}")
            return fabfile.run_command(cmd, config=kw.get("env_config"))

    def drain(self, **kw):
        """
        To drain all daemons from specified host.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
                host_name(str): name of host.
                force(bool): force drain.
        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        host_name = kw.get("host_name", "")
        force = "--force" if kw.get("force") else ""
        cmd = self.base_cmd + f" drain {host_name} {force}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def rm(self, **kw):
        """
        To remove host.
        Args:
          kw (Dict): Key/value pairs that needs to be provided to the installer.
          Example:
            Supported keys:
                host_name(str): name of host.
                force(bool): Whether its a forced operation or not(Optional).
                offline(bool): if a host is offline(Optional).

        Returns:
          Dict(str)
            A mapping of host strings of the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        host_name = kw_copy.pop("host_name", "")
        cmd = self.base_cmd + f" rm {host_name}" + core_utils.build_cmd_args(kw=kw_copy)
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))



# -*- coding: utf-8 -*-
try:
    from setuptools import find_packages, setup
except ImportError:
    from ez_setup import use_setuptools

    use_setuptools()
    from setuptools import find_packages, setup

setup(
    name="RhcsQeSdk",
    version="0.0.7",
    description="RHCS QE SDK Library",
    url="https://gitlab.cee.redhat.com/rhcs-qe/rhcs-qe-sdk",
    author="kasiramkompella",
    author_email="kasiramkompella@gmail.com",
    install_requires=["fabric3"],
    long_description_content_type="text/markdown",
    zip_safe=True,
    include_package_data=True,
    packages=find_packages("src", exclude=["ez_setup"]),
    package_dir={"": "src"},
)



import logging
from copy import deepcopy

import src.RhcsQeSdk.core.cli.fabfile as fabfile
from src.RhcsQeSdk.core.cli.rbd.mirror.peer import Peer
from src.RhcsQeSdk.core.utilities import core_utils

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(
    "%(asctime)s - %(levelname)s - %(name)s:%(lineno)d - %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


class Pool:
    """This module provides CLI interface to manage rbd mirror pool commands."""

    def __init__(self, base_cmd):
        self.base_cmd = base_cmd + " pool"
        self.peer = Peer(self.base_cmd)

    def demote(self, **kw):
        """Wrapper for rbd mirror pool demote.

        Command is used to demote all primary images in the pool.

        Args:
            kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_name: Name of the pool.
                namespace: Name of the namespace.
                pool_spec: pool-name/namespace.
        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        pool_spec = kw_copy.pop("pool_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} demote {pool_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def disable(self, **kw):
        """Wrapper for rbd mirror pool disable.

        Command is used to disable RBD mirroring by default within a pool.

        Args:
            kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_name: Name of the pool.
                namespace: Name of the namespace.
                pool_spec: pool-name/namespace.

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")
        kw_copy = deepcopy(kw)
        pool_spec = kw_copy.pop("pool_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} disable {pool_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def enable(self, **kw):
        """Wrapper for rbd mirror pool enable.

        Command is used to enable RBD mirroring by default within a pool.

        Args:
            kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_name: Name of the pool.
                namespace: Name of the namespace.
                pool_spec: pool-name/namespace.
                mode: mirror mode [image or pool]
                site-name: Name of the site.

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_name = kw_copy.pop("pool_name", "")
        mode = kw_copy.pop("mode", "")
        site_name = kw_copy.pop("site_name", "")

        cmd = (
            f"{self.base_cmd} enable --site-name {site_name} --pool {pool_name} {mode}"
        )
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def info(self, **kw):
        """Wrapper for rbd mirror pool info.

        Command is used to show information about the pool mirroring configuration.
        Args:
            kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_name: Name of the pool.
                namespace: Name of the namespace.
                pool_spec: pool-name/namespace.
                format: output format (plain, json, or xml) [default: plain]
                pretty-format: True - pretty formatting (json and xml)
                all: True - All information

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_spec = kw_copy.pop("pool_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} info {pool_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def promote(self, **kw):
        """Wrapper for rbd mirror pool promote.

        Command is used to promote a secondary image to primary.
        Args:
            kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_name: Name of the pool.
                namespace: Name of the namespace.
                pool_spec: pool-name/namespace.
                force: True - Bool value to force promote the image.

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_spec = kw_copy.pop("pool_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)
        cmd = f"{self.base_cmd} promote {pool_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))

    def status(self, **kw):
        """Wrapper for rbd mirror pool status.

        Command is used to display the mirroring status of pool.
        Args:
            kw: Key value pair of method arguments
            Example::
            Supported keys:
                pool_name: Name of the pool.
                namespace: Name of the namespace.
                pool_spec: pool-name/namespace.
                format: output format (plain, json, or xml) [default: plain]
                pretty-format: pretty formatting (json and xml)
                verbose: Bool Value - display status of all images.

        Returns:
            Dict(str):
            A mapping of host strings to the given task's return value for that host's execution run.
        """
        kw = kw.get("kw")

        kw_copy = deepcopy(kw)
        pool_spec = kw_copy.pop("pool_spec", "")
        cmd_args = core_utils.build_cmd_args(kw=kw_copy)

        cmd = f"{self.base_cmd} status {pool_spec}{cmd_args}"
        logger.info(f"Running command {cmd}")
        return fabfile.run_command(cmd, config=kw.get("env_config"))
